"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.P521Jwk = void 0;
exports.isValidP521JwkPublicKey = isValidP521JwkPublicKey;
const error_1 = require("../../../error");
const utils_1 = require("../../../utils");
const KeyType_1 = require("../../KeyType");
const jwa_1 = require("../jwa");
const alg_1 = require("../jwa/alg");
const Jwk_1 = require("./Jwk");
const ecCompression_1 = require("./ecCompression");
const validate_1 = require("./validate");
class P521Jwk extends Jwk_1.Jwk {
    constructor({ x, y }) {
        super();
        const xAsBytes = typeof x === 'string' ? Uint8Array.from(utils_1.TypedArrayEncoder.fromBase64(x)) : x;
        const yAsBytes = typeof y === 'string' ? Uint8Array.from(utils_1.TypedArrayEncoder.fromBase64(y)) : y;
        this._x = xAsBytes;
        this._y = yAsBytes;
    }
    get kty() {
        return jwa_1.JwaKeyType.EC;
    }
    get crv() {
        return jwa_1.JwaCurve.P521;
    }
    get keyType() {
        return P521Jwk.keyType;
    }
    get supportedEncryptionAlgorithms() {
        return P521Jwk.supportedEncryptionAlgorithms;
    }
    get supportedSignatureAlgorithms() {
        return P521Jwk.supportedSignatureAlgorithms;
    }
    get x() {
        return utils_1.TypedArrayEncoder.toBase64URL(this._x);
    }
    get y() {
        return utils_1.TypedArrayEncoder.toBase64URL(this._y);
    }
    /**
     * Returns the uncompressed public key of the P-521 JWK.
     */
    get publicKey() {
        return new Uint8Array([ecCompression_1.PREFIX_UNCOMPRESSED, ...this._x, ...this._y]);
    }
    /**
     * Returns the compressed public key of the P-521 JWK.
     */
    get publicKeyCompressed() {
        return (0, ecCompression_1.compressECPoint)(this._x, this._y);
    }
    toJson() {
        return Object.assign(Object.assign({}, super.toJson()), { crv: this.crv, x: this.x, y: this.y });
    }
    static fromJson(jwk) {
        if (!isValidP521JwkPublicKey(jwk)) {
            throw new Error("Invalid 'P-521' JWK.");
        }
        return new P521Jwk({
            x: jwk.x,
            y: jwk.y,
        });
    }
    static fromPublicKey(publicKey) {
        if ((0, ecCompression_1.isValidCompressedPublicKey)(publicKey, this.keyType)) {
            const expanded = (0, ecCompression_1.expand)(publicKey, this.keyType);
            const x = expanded.slice(1, expanded.length / 2 + 1);
            const y = expanded.slice(expanded.length / 2 + 1);
            return new P521Jwk({ x, y });
        }
        if ((0, ecCompression_1.isValidUncompressedPublicKey)(publicKey, this.keyType)) {
            const x = publicKey.slice(1, publicKey.length / 2 + 1);
            const y = publicKey.slice(publicKey.length / 2 + 1);
            return new P521Jwk({ x, y });
        }
        throw new error_1.CredoError(`${this.keyType} public key is neither a valid compressed or uncompressed key. Key prefix '${publicKey[0]}', key length '${publicKey.length}'`);
    }
}
exports.P521Jwk = P521Jwk;
P521Jwk.supportedEncryptionAlgorithms = [];
P521Jwk.supportedSignatureAlgorithms = [alg_1.JwaSignatureAlgorithm.ES512];
P521Jwk.keyType = KeyType_1.KeyType.P521;
function isValidP521JwkPublicKey(jwk) {
    return ((0, validate_1.hasKty)(jwk, jwa_1.JwaKeyType.EC) &&
        (0, validate_1.hasCrv)(jwk, jwa_1.JwaCurve.P521) &&
        (0, validate_1.hasX)(jwk) &&
        (0, validate_1.hasY)(jwk) &&
        (0, validate_1.hasValidUse)(jwk, {
            supportsEncrypting: true,
            supportsSigning: true,
        }));
}
//# sourceMappingURL=P521Jwk.js.map